<?php
/*
Plugin Name: ChloroAttachment for WordPress
Plugin URI: http://martincl2.me/archives/453
Description: SAE Storage 附件上传，代替原版WordPress的附件上传功能以兼容SAE环境
Version: 1.0-mu
Author: Martin Chloride
Author URI: http://martincl2.me
*/

/*  Copyright 2013  Martin Chloride  (email : i@martincl2.me)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

namespace SAE\attachment;

\register_activation_hook( __FILE__, 'SAE\attachment\install'); 
\add_action('init', 'SAE\attachment\init');
\add_action('admin_menu', 'SAE\attachment\menu');
\add_filter('plugin_action_links', 'SAE\attachment\link',9999,2);

function init(){
	$action = \basename($_SERVER['SCRIPT_FILENAME']);
	switch ($action)
	{
		case 'update.php':
		case 'update-core.php':
		case 'install.php':
			break;
			
		default:
			$storage_domain = \get_option('storage_domain');
			$storage_directory = \get_option('storage_directory');
			
			if($storage_domain === false){
				\add_option('storage_domain', 'wordpress');
				$storage_domain = 'wordpress';
			}
			
			if($storage_directory === false)
			{
			 	\add_option('storage_directory', 'uploads');
			 	$storage_directory = 'uploads';
			}
			
			define('SAE_STORAGE', $storage_domain);
			define('SAE_UPLOAD_DIR', $storage_directory);
			
			define('SAE_DIR', 'saestor://'.SAE_STORAGE);
			define('SAE_URL', 'http://'.$_SERVER['HTTP_APPNAME'].'-'.SAE_STORAGE.'.stor.sinaapp.com/'.SAE_UPLOAD_DIR);
			define('SAE_TMP_DIR_STORAGE', \SAE_TMP_PATH.'storage');
			
			\add_filter('upload_dir', 'SAE\attachment\upload_dir');
			\add_filter('wp_handle_upload', 'SAE\attachment\handle_upload');
			\add_filter('wp_create_file_in_uploads', 'SAE\attachment\upload_all');
			\add_filter('wp_generate_attachment_metadata', 'SAE\attachment\upload_thumb', 999);
			\add_filter('wp_delete_file', 'SAE\attachment\delete_file');
			break;
	}
	
	return;
}

function menu(){
	add_options_page('SAE Storage 参数配置', 'SAE Storage 附件上传', 'administrator', 'config-chloroattachment', 'SAE\attachment\config_menu');
	
	return;
}

function install(){
	if(!\get_option('storage_domain')){
		\add_option('storage_domain', 'wordpress');
	}
	if(!\get_option('storage_directory')){
		\add_option('storage_directory', 'uploads');
	}
}

function link($links, $file) {
	if ($file == 'sae-attachment/chloroattachment.php' && function_exists("admin_url")) {
		$settings_link[] = '<a href="' . admin_url('options-general.php?page=config-chloroattachment' ). '">' . __('Settings') . '</a>';
		$links = array_merge($settings_link, $links);
	}
	return $links;
}


function upload_dir($uploads){
	//Modify where attachments should upload and where users should access to
	$uploads['basedir'] = SAE_TMP_DIR_STORAGE;
	$uploads['baseurl'] = SAE_URL;
	$uploads['path'] = SAE_TMP_DIR_STORAGE.$uploads['subdir'];
	$uploads['url'] = SAE_URL.$uploads['subdir'];
	
	return $uploads;
}

function handle_upload($file){
	$s = new \SaeStorage();
	
	//Set unique filename
	$filename = \basename($file['file']);
	$dirname = \dirname($file['file']);
	$new_loc = $dirname.'/'.unique_filename(SAE_UPLOAD_DIR.\str_replace(SAE_TMP_DIR_STORAGE.'/', '/', $dirname), $filename);
	\rename($file['file'], $new_loc);
	$file['file'] = $new_loc;
	
	$filename = \str_replace(SAE_TMP_DIR_STORAGE.'/', '', $file['file']);
	
	//Upload Attachment
	$s->upload(SAE_STORAGE, SAE_UPLOAD_DIR.'/'.$filename, $file['file']);
	
	return $file;
}

function upload_all($file = null, $aid = null){
	upload_directory(SAE_TMP_DIR_STORAGE);
	
	return $file;
}

function upload_directory($directory){
	if(substr($directory, -1, 1) === '/')
		$directory = substr($directory, 0, strlen($directory)-1);
		
	if($handle = opendir($directory)){
		while(false !== ($file = readdir($handle))){
			if($file !== '.' && $file !== '..'){
				if(is_dir($directory.'/'.$file)){
					upload_directory($directory.'/'.$file);
				}else{
					upload_file($directory.'/'.$file);
				}
			}
		}
	}
}

function upload_file($file, $clean = true){
	$s = new \SaeStorage();
	
	$filename = \str_replace(SAE_TMP_DIR_STORAGE.'/', '', $file);
	
	//Upload Attachment
	$s->upload(SAE_STORAGE, SAE_UPLOAD_DIR.'/'.$filename, $file);
	
}

function upload_thumb($metadata){
	//All types of file will trigger this filter
	//But only images could have contents in array $metadata
	if(isset($metadata['sizes'])){
		$dir = SAE_TMP_DIR_STORAGE.'/'.dirname($metadata['file']);
		$s = new \SaeStorage();
		foreach($metadata['sizes'] as $size_name => $image_meta){
			$file = $image_meta['file'];
			$s->upload(SAE_STORAGE, SAE_UPLOAD_DIR.'/'.dirname($metadata['file']).'/'.$image_meta['file'], $dir.'/'.$image_meta['file']);
		}
	}
	return $metadata;
}

function delete_file($file){
	$url_pos = \strpos($file, '/storage/');
	if($url_pos !== false){
		$file = \substr($file, $url_pos + 9);
	}
	$file = SAE_UPLOAD_DIR.'/'.$file;
	
	$s = new \SaeStorage();
	$result = $s->delete(SAE_STORAGE, $file);
	return $file;
}

function unique_filename( $dir, $filename ) {
	// sanitize the file name before we begin processing
	$filename = \sanitize_file_name($filename);

	// separate the filename into a name and extension
	$info = \pathinfo($filename);
	$ext = !empty($info['extension']) ? '.' . $info['extension'] : '';
	$name = \basename($filename, $ext);

	// edge case: if file is named '.ext', treat as an empty name
	if ( $name === $ext )
		$name = '';

	$number = '';

	// change '.ext' to lower case
	if ( $ext && \strtolower($ext) != $ext ) {
		$ext2 = \strtolower($ext);
		$filename2 = \preg_replace( '|' . \preg_quote($ext) . '$|', $ext2, $filename );

		// check for both lower and upper case extension or image sub-sizes may be overwritten
		while ( file_exists($dir . "/$filename") || file_exists($dir . "/$filename2") ) {
			$new_number = $number + 1;
			$filename = \str_replace( "$number$ext", "$new_number$ext", $filename );
			$filename2 = \str_replace( "$number$ext2", "$new_number$ext2", $filename2 );
			$number = $new_number;
		}
		return $filename2;
	}

	while ( file_exists( $dir . "/$filename" ) ) {
		if ( '' == "$number$ext" )
			$filename = $filename . ++$number . $ext;
		else
			$filename = \str_replace( "$number$ext", ++$number . $ext, $filename );
	}

	return $filename;
}

function file_exists($filename){
	$s = new \SaeStorage();
	return $s->fileExists(SAE_STORAGE, $filename);
}

function config_menu(){
?>
	<div>
		<h2>ChloroAttachment 配置</h2>
		<table>
			<tr>
				<td>Storage Domain</td>
				<td>
					<form method="post" action="options.php">
						<?php wp_nonce_field('update-options'); ?>
			
						<input type="input" name="storage_domain" value="<?php echo get_option('storage_domain'); ?>" />
						</textarea>
							
						<input type="hidden" name="action" value="update" />
						<input type="hidden" name="page_options" value="storage_domain" />
			
						<input type="submit" value="保存" class="button-primary" />
					</form>
				</td>
			</tr>
			<tr>
				<td>Storage 路径</td>
				<td>
					<form method="post" action="options.php">
						<?php wp_nonce_field('update-options'); ?>
			
						<input type="input" name="storage_directory" value="<?php echo get_option('storage_directory'); ?>" />
						</textarea>
							
						<input type="hidden" name="action" value="update" />
						<input type="hidden" name="page_options" value="storage_directory" />
			
						<input type="submit" value="保存" class="button-primary" />
					</form>
				</td>
			</tr>
		</table>
	</div>
<?php
}

?>